#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class to represent a named.conf key
#
#

import os
import os.path
import sys

if os.path.exists('../bind2nsd/Config.py'):
   sys.path.append('../bind2nsd')
   from Utils import *
else:
   from bind2nsd.Utils import *


class Key:

   def __init__(self, name):
      self.name = name
      self.algorithm = ''
      self.secret = ''
      self.ipaddrs = []
      return

   def dump(self):
      report_info('=> Key:')
      report_info('   algorithm = %s' % (self.algorithm))
      report_info('   name      = %s' % (self.name))
      report_info('   secret    = %s' % (self.secret))
      report_info('   ipaddrs   = %s' % (str(self.ipaddrs)))
      return

   def setName(self, val):
      self.name = val
      return

   def getName(self):
      return self.name

   def setAlgorithm(self, val):
      self.algorithm = val
      return

   def getAlgorithm(self):
      return self.algorithm

   def setSecret(self, val):
      self.secret = val
      return

   def getSecret(self):
      return self.secret

   def addIpAddr(self, addr):
      if addr not in self.ipaddrs:
         self.ipaddrs.append(addr)
      return
      
   def getIpAddrs(self):
      return self.ipaddrs




#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class to represent a zone file
#
#

import os
import os.path
import sys

if os.path.exists('../bind2nsd/Config.py'):
   sys.path.append('../bind2nsd')
   from Utils import *
else:
   from bind2nsd.Utils import *


class Zone:

   def __init__(self, name):
      self.name = name
      self.file = ''
      self.type = ''
      self.masters = []		# empty unless we're a slave zone
      self.allow_transfer = []
      self.also_notify = []
      self.allow_notify = []
      return

   def dump(self):
      report_info('=> Zone:')
      report_info('   name = %s' % (self.name))
      report_info('   file = %s' % (self.file))
      report_info('   type = %s' % (self.type))
      report_info('   masters = %s' % (str(self.masters)))
      report_info('   allow_notify = %s' % (str(self.allow_notify)))
      report_info('   allow_transfer = %s' % (str(self.allow_transfer)))
      report_info('   also_notify = %s' % (str(self.also_notify)))
      return

   def setName(self, name):
      self.name = name
      return

   def getName(self):
      return self.name

   def setFile(self, file):
      self.file = file
      return

   def getFile(self):
      return self.file

   def setType(self, type):
      self.type = type
      return

   def getType(self):
      return self.type

   def addMaster(self, quad):
      self.masters.append(quad)
      return

   def setMasters(self, list):
      self.masters = list
      return

   def getMasters(self):
      return self.masters

   def addAllowTransfer(self, quad):
      self.allow_transfer.append(quad)
      return

   def setAllowTransfers(self, list):
      self.allow_transfer = list
      return

   def getAllowTransfer(self):
      return self.allow_transfer

   def addAlsoNotify(self, quad):
      self.also_notify.append(quad)
      return

   def setAlsoNotify(self, list):
      self.also_notify = list
      return

   def getAlsoNotify(self):
      return self.also_notify

   def addAllowNotify(self, quad):
      self.allow_notify.append(quad)
      return

   def setAllowNotify(self, list):
      self.allow_notify = list
      return

   def getAllowNotify(self):
      return self.allow_notify




#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class containing the parser for BIND configuration files
#
#

import os
import os.path
import sys

if os.path.exists('../bind2nsd/Config.py'):
   sys.path.append('../bind2nsd')
   from NamedConf import *
   from Zone import *
   from Key import *
   from Tokenizer import *
   from Utils import *
else:
   from bind2nsd.NamedConf import *
   from bind2nsd.Zone import *
   from bind2nsd.Key import *
   from bind2nsd.Tokenizer import *
   from bind2nsd.Utils import *


#-- the following options {} clauses are currently currently ignored,
#   typically because they are not implemented or not applicable to NSD
IGNORED_OPTIONS = [ 'cleaning-interval',
                    'datasize',
                    'interface-interval',
		    'max-cache-size',
                    'stacksize',
                  ]

class Parser:

   def __init__(self, named):
      self.named = named
      return

   def handle_include(self, tokens):
      (ttype, val, curline) = tokens.get()
      if ttype == T_STRING:
         fname = val.replace('"', '')
         self.named.includes.append(fname)
         report_info('   include file "%s"...' % (fname))
         if os.path.exists(fname):
            self.parse(Tokenizer(fname))
         else:
            report_error('? missing include file "%s"' % (fname))
      else:
         bail('? dude.  where is the filename string after "include"?')
      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? need a semicolon to end the "include" (%d: %s)' % (ttype, val))
      return
   
   def handle_key(self, tokens):
      (ttype, val, curline) = tokens.get()
      if ttype == T_STRING:
         name = val.replace('"', '')
         key = Key(name)
         (ttype, val, curline) = tokens.get()
         if ttype != T_LBRACE:
            bail('? need opening brace for "key" defn (%d: %s)' % (ttype, val))
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE:
            if ttype == T_KEYWORD and val == 'algorithm':
               (ttype, val, curline) = tokens.get()
 	       if ttype == T_KEYWORD or ttype == T_NAME:
	          key.setAlgorithm(val)
	       else:
	          bail('? bogosity after "algorithm" (%d: %s)' % (ttype, val))
	    elif ttype == T_KEYWORD and val == 'secret':
	       (ttype, val, curline) = tokens.get()
	       if ttype == T_STRING:
	          s = val.replace('"','')
		  key.setSecret(s)
	       else:
	          bail('? bogosity after "secret" (%d: %s)' % (ttype, val))
	    elif ttype == T_SEMI:
	       pass
	    (ttype, val, curline) = tokens.get()
      else:
         bail('? need to have a string after "key" (%d: %s)' % (ttype, val))

      self.named.addKey(key)
      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? need a semicolon to end the "key" (%d: %s)' % (ttype, val))
      return
   
   def handle_server(self, tokens):
      (ttype, val, curline) = tokens.get()
      if ttype == T_QUAD:
         ipaddr = val
         (ttype, val, curline) = tokens.get()
         if ttype != T_LBRACE:
            bail('? need opening brace for "server" (%d: %s)' % (ttype, val))
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE:
            if ttype == T_KEYWORD and val == 'keys':
               (ttype, val, curline) = tokens.get()
 	       if ttype == T_LBRACE:
	          nlist = self.name_list(val, tokens)
		  for ii in nlist:
		     self.named.keys[ii].addIpAddr(ipaddr)
	       else:
	          bail('? bogosity after "keys" (%d: %s)' % (ttype, val))
	    (ttype, val, curline) = tokens.get()
      else:
         bail('? need to have an IP address after "server" (%d: %s)' % \
	      (ttype, val))

      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? need a semicolon to end the "server" (%d: %s)' % (ttype, val))
      return
   
   def name_list(self, name, tokens):
      nlist = []
      (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE or ttype == T_LPAREN:
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE and ttype != T_RPAREN:
            if ttype == T_NAME:
	       nlist.append(val)
            elif ttype == T_SEMI or ttype == T_COMMENT:
               pass
            else:
               bail('? was expecting a name in "%s" (%d, %d: %s)' % \
	            (name, ttype, curline, val.strip()))
            (ttype, val, curline) = tokens.get()
         (ttype, val, curline) = tokens.get()
         if ttype != T_SEMI:
            bail('? junk after closing brace of "%s" section' % (name))

      return nlist

   def quad_list(self, name, tokens):
      qlist = []
      (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE or ttype == T_LPAREN:
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE and ttype != T_RPAREN:
            if ttype == T_QUAD:
	       qlist.append(val)
            elif ttype == T_SEMI or ttype == T_COMMENT:
               pass
            else:
               bail('? was expecting a quad in "%s" (%d, %d: %s)' % \
	            (name, ttype, curline, val.strip()))
            (ttype, val, curline) = tokens.get()
         (ttype, val, curline) = tokens.get()
         if ttype != T_SEMI:
            bail('? junk after closing brace of "%s" section' % (name))

      return qlist

   def quad_list_with_keys(self, name, tokens, named):
      qlist = []
      (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE or ttype == T_LPAREN:
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE and ttype != T_RPAREN:
            if ttype == T_QUAD:
	       qlist.append(val)
            elif ttype == T_SEMI or ttype == T_COMMENT:
               pass
	    elif ttype == T_KEYWORD and val == 'key':
               (ttype, val, curline) = tokens.get()
	       if ttype == T_NAME:
	          if val in named.getKeys():
		     qlist.append(val)
	       else:
                  bail('? was expecting a key name in "%s" (%d, %d: %s)' % \
	               (name, ttype, curline, val.strip()))
            else:
               bail('? was expecting a quad in "%s" (%d, %d: %s)' % \
	            (name, ttype, curline, val.strip()))
            (ttype, val, curline) = tokens.get()
         (ttype, val, curline) = tokens.get()
         if ttype != T_SEMI:
            bail('? junk after closing brace of "%s" section' % (name))

      return qlist

   def skip_value(self, name, tokens):
      (ttype, val, curline) = tokens.get()
      if ttype == T_NAME or ttype == T_STRING:
         pass
      else:
         bail('? need something after "%s" keyword (%d: %s)' % 
	      (name, ttype, val))
      return

   def handle_options(self, tokens):
      report_info('=> processing server options')
      optsDone = False
      depth = 0
      (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE:
         depth += 1
      else:
         bail('? urk.  bad info after "options" keyword.')
      (ttype, val, curline) = tokens.get()

      while not optsDone:
         if ttype == T_KEYWORD and val == 'directory':
            (ttype, val, curline) = tokens.get()
            if ttype == T_STRING:
               self.named.options.setDirectory(val)
	    else:
               bail('? need string after "directory" keyword')

	 elif ttype == T_KEYWORD and val == 'dump-file':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_STRING:
	       self.named.options.setDumpFile(val)
	    else:
	       bail('? need string after "dump-file" keyword')

	 elif ttype == T_KEYWORD and val == 'pid-file':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_STRING:
	       self.named.options.setPidFile(val)
	    else:
	       bail('? need string after "pid-file" keyword')

	 elif ttype == T_KEYWORD and val == 'coresize':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_NAME or ttype == T_KEYWORD:
	       self.named.options.setCoresize(val)
	    else:
	       bail('? need name after "coresize" keyword (%d: %s)' % \
	            (ttype, val))

	 elif ttype == T_KEYWORD and val == 'version':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_STRING:
	       self.named.options.setVersion(val)
	    else:
	       bail('? need string after "version" keyword')

	 elif ttype == T_KEYWORD and val == 'transfer-format':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_NAME or ttype == T_KEYWORD:
	       self.named.options.setTransferFormat(val)
	    else:
	       bail('? need name after "transfer-format" keyword (%d: %s)' % \
	            (ttype, val))

         elif ttype == T_KEYWORD and val == 'statistics-file':
            (ttype, val, curline) = tokens.get()
            if ttype == T_STRING:
               self.named.options.setStatisticsFile(val)
            else:
  	       bail('? need string after "statistics-file" keyword')

	 elif ttype == T_KEYWORD and val == 'allow-transfer':
	    qlist = self.quad_list_with_keys(val, tokens, self.named)
	    self.named.options.setAllowTransfer(qlist)

	 elif ttype == T_KEYWORD and val == 'allow-notify':
	    qlist = self.quad_list(val, tokens)
	    self.named.options.setAllowNotify(qlist)

	 elif ttype == T_KEYWORD and val == 'also-notify':
	    qlist = self.quad_list(val, tokens)
	    self.named.options.setAlsoNotify(qlist)

	 elif ttype == T_KEYWORD and val == 'allow-recursion':
	    qlist = self.quad_list(val, tokens)
	    self.named.options.setAllowRecursion(qlist)

         elif ttype == T_KEYWORD and val == 'check-names':
            (ttype, val, curline) = tokens.get()
            info = []
            while ttype != T_SEMI:
  	       info.append(val)
               (ttype, val, curline) = tokens.get()
            self.named.options.addCheckNames(' '.join(info))
   
         elif ttype == T_KEYWORD and val in IGNORED_OPTIONS:
	    self.skip_value(val, tokens)

	 elif ttype == T_KEYWORD and val == 'recursive-clients':
	    (ttype, val, curline) = tokens.get()
	    if ttype == T_QUAD:		# well, it's a number, actually...
	       self.named.options.setRecursiveClients(val)
	    else:
	       bail('? need value after "recursive-clients" keyword (%d: %s)'
	            % (ttype, val))

         elif ttype == T_RBRACE:
            optDone = True
            break
         (ttype, val, curline) = tokens.get()

      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? um, junk after closing brace of "options" section')
   
      return

   def handle_zone(self, tokens):
      (ttype, val, curline) = tokens.get()
      if ttype != T_STRING:
         bail('? expected string for zone name (%d: %s)' % (ttype, val))
      zname = val.replace('"','')
      zone = Zone(zname)
   
      (ttype, val, curline) = tokens.get()
      if ttype == T_KEYWORD and (val == 'in' or val == 'IN'):
         (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE or ttype == T_LPAREN:
         (ttype, val, curline) = tokens.get()
         while ttype != T_RBRACE and ttype != T_RPAREN:
            if ttype == T_KEYWORD and val == 'type':
               (ttype, val, curline) = tokens.get()
  	       info = []
  	       while ttype == T_KEYWORD or ttype == T_NAME:
  	          info.append(val)
                  (ttype, val, curline) = tokens.get()
  	       zone.setType(' '.join(info))

            elif ttype == T_KEYWORD and val == 'file':
               (ttype, val, curline) = tokens.get()
	       if ttype != T_STRING:
	          bail('? eh?  no string for zone file name? (%d: %s)' % \
		       (ttype, val))
	       fname = val.replace('"','')
	       zone.setFile(fname)

	    elif ttype == T_KEYWORD and val == 'masters':
	       qlist = self.quad_list(val, tokens)
	       zone.setMasters(qlist)

	    elif ttype == T_KEYWORD and val == 'allow-transfer':
	       qlist = self.quad_list_with_keys(val, tokens, self.named)
	       zone.setAllowTransfers(qlist)

	    elif ttype == T_KEYWORD and val == 'also-notify':
	       qlist = self.quad_list(val, tokens)
	       zone.setAlsoNotify(qlist)

	    elif ttype == T_SEMI:
	       pass

	    else:
	       bail('? bugger.  do NOT grok "%s" yet (%d: %s @ line %s)' % \
	            (val, ttype, val, curline))

	    (ttype, val, curline) = tokens.get()

	 self.named.addZone(zone)

      else:
         bail('? expected left brace to start zone definition (%d: %s)' % \
	      (ttype, val))

      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? need a semicolon to end the "zone" (%d: %s)' % (ttype, val))

      return
   
   def ignore_section(self, name, tokens):
      report_info('=> ignoring %s' % (name))
      done = False
      depth = 0
      (ttype, val, curline) = tokens.get()
      if ttype == T_LBRACE:
         depth += 1
      else:
         bail('? urk.  bad info after "%s" keyword.' % (name))
      (ttype, val, curline) = tokens.get()
      while not done:
         if ttype == T_LBRACE:
            depth += 1
         elif ttype == T_RBRACE:
            depth -= 1
            if depth == 0:
               break
            else:
               pass
         (ttype, val, curline) = tokens.get()
      (ttype, val, curline) = tokens.get()
      if ttype != T_SEMI:
         bail('? um, junk after closing brace of "%s" section' % (name))
      return

   def parse(self, tokens):
      (ttype, val, curline) = tokens.get()
      while ttype != T_EOF:
         if ttype == T_COMMENT:
            pass

         elif ttype == T_KEYWORD and val == 'controls':
	    self.ignore_section(val, tokens)
   
         elif ttype == T_KEYWORD and val == 'key':
	    self.handle_key(tokens)

         elif ttype == T_KEYWORD and val == 'include':
	    self.handle_include(tokens)

         elif ttype == T_KEYWORD and val == 'logging':
	    self.ignore_section(val, tokens)
   
         elif ttype == T_KEYWORD and val == 'lwres':
	    self.ignore_section(val, tokens)
   
         elif ttype == T_KEYWORD and val == 'options':
	    self.handle_options(tokens)

         elif ttype == T_KEYWORD and val == 'server':
	    self.handle_server(tokens)

         elif ttype == T_KEYWORD and val == 'trusted-keys':
	    self.ignore_section(val, tokens)

         elif ttype == T_KEYWORD and val == 'view':
	    self.ignore_section(val, tokens)

         elif ttype == T_KEYWORD and val == 'zone':
	    self.handle_zone(tokens)

         else:
            bail('? ew.  what _is_ this? -> %2d: %s @ line %d' \
	         % (ttype, val, curline))
         (ttype, val, curline) = tokens.get()
   
      return




#
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#    installation/setup script for bind2nsd
#

from distutils.core import setup

setup(name = 'bind2nsd',
      version = '0.5.0',
      author = 'Secure64 Software Corporation',
      author_email = 'support@secure64.com',
      maintainer = 'Al Stone',
      maintainer_email = 'ahs3@secure64.com',
      url = 'http://www.secure64.com',
      description = 'automatic named/nsd translation and synchronization',
      packages = [ 'bind2nsd' ],
      scripts = [ 'scripts/bind2nsd', 'scripts/s64-sync',
                  'scripts/s64-mkpw', ],
      data_files = [ ('/usr/share/doc/bind2nsd', ['./README', './TODO']),
                     ('/etc/bind2nsd', ['etc/bind2nsd.conf']),
		   ]
     )




#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class to represent a named.conf file
#

import os
import os.path
import sys

if os.path.exists('../bind2nsd/Config.py'):
   sys.path.append('../bind2nsd')
   from Config import *
   from Parser import *
   from Zone import *
   from Utils import *
else:
   from bind2nsd.Config import *
   from bind2nsd.Parser import *
   from bind2nsd.Zone import *
   from bind2nsd.Utils import *


class Options:
   def __init__(self):
      self.allow_notify = []
      self.allow_recursion = []
      self.allow_transfer = []
      self.also_notify = []
      self.check_names = []
      self.coresize = ""
      self.directory = ""
      self.dump_file = ""
      self.pid_file = ""
      self.recursive_clients = ""
      self.statistics_file = ""
      self.transfer_format = ""
      self.version = ""
      return

   def dump(self):
      report_info('=> Options:')
      report_info('   allow-notify:      %s' % (self.allow_notify))
      report_info('   allow-recursion:   %s' % (self.allow_recursion))
      report_info('   allow-transfer:    %s' % (self.allow_transfer))
      report_info('   also-notify:       %s' % (self.also_notify))
      report_info('   check-names:       %s' % (str(self.check_names)))
      report_info('   coresize:          %s' % (self.coresize))
      report_info('   directory:         %s' % (self.directory))
      report_info('   dump-file:         %s' % (self.dump_file))
      report_info('   pid-file:          %s' % (self.pid_file))
      report_info('   recursive-clients: %s' % (self.recursive_clients))
      report_info('   statistics-file:   %s' % (self.statistics_file))
      report_info('   transfer-format:   %s' % (self.transfer_format))
      report_info('   version:           %s' % (self.version))
      return

   def setDirectory(self, name):
      self.directory = name
      return

   def getDirectory(self):
      return self.directory

   def setDumpFile(self, name):
      self.dump_file = name
      return

   def getDumpFile(self):
      return self.dump_file

   def setPidFile(self, name):
      self.pid_file = name
      return

   def getPidFile(self):
      return self.pid_file

   def setStatisticsFile(self, name):
      self.statistics_file = name
      return

   def getStatisticsFile(self):
      return self.statistics_file

   def addAllowNotify(self, quad):
      self.allow_notify.append(quad)
      return

   def setAllowNotify(self, list):
      self.allow_notify = list
      return

   def getAllowNotify(self):
      return self.allow_notify

   def addAllowTransfer(self, quad):
      self.allow_transfer.append(quad)
      return

   def setAllowTransfer(self, list):
      self.allow_transfer = list
      return

   def getAllowTransfer(self):
      return self.allow_transfer

   def addAlsoNotify(self, quad):
      self.also_notify.append(quad)
      return

   def setAlsoNotify(self, list):
      self.also_notify = list
      return

   def getAlsoNotify(self):
      return self.also_notify

   def addAllowRecursion(self, quad):
      self.allow_recursion.append(quad)
      return

   def setAllowRecursion(self, list):
      self.allow_recursion = list
      return

   def getAllowRecursion(self):
      return self.allow_recursion

   def addCheckNames(self, val):
      self.check_names.append(val)
      return

   def getCheckNames(self):
      return self.check_names

   def setCoresize(self, val):
      self.coresize = val
      return

   def getCoresize(self):
      return self.coresize

   def setTransferFormat(self, val):
      self.transfer_format = val
      return

   def getTransferFormat(self):
      return self.transfer_format

   def setVersion(self, val):
      self.version = val
      return

   def getVersion(self):
      return self.version

   def setRecursiveClients(self, val):
      self.recursive_clients = val
      return

   def getRecursiveClients(self):
      return self.recursive_clients


class NamedConf:

   def __init__(self, fname):
      self.fname = fname
      self.options = Options()
      self.includes = []
      self.zones = {}
      self.keys = {}

      parser = Parser(self)
      parser.parse(Tokenizer(self.fname))
      return

   def dump(self):
      report_info('=> NamedConf: dumping data from \"%s\"...' % (self.fname))
      self.options.dump()
      report_info('=> NamedConf: list of includes...')
      report_info(str(self.includes))

      report_info('=> NamedConf: zone info...')
      zlist = self.zones.keys()
      zlist.sort()
      for ii in zlist:
         self.zones[ii].dump()
   
      report_info('=> NamedConf: key info...')
      klist = self.keys.keys()
      klist.sort()
      for ii in klist:
         self.keys[ii].dump()

      return

   def getOptions(self):
      return self.options

   def addZone(self, zone):
      name = zone.getName()
      self.zones[name] = zone
      return

   def getZones(self):
      return self.zones

   def getZone(self, name):
      if name in self.zones:
         return self.zones[name]
      else:
         return None

   def addKey(self, key):
      name = key.getName()
      self.keys[name] = key
      return

   def getKey(self, name):
      if name in self.keys:
         return self.keys[name]
      else:
         return None

   def getKeys(self):
      return self.keys




#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class to represent a nsd.conf file
#
#

import os.path
import sys

if os.path.exists('../bind2nsd/Config.py'):
   sys.path.append('../bind2nsd')
   from Utils import *
else:
   from bind2nsd.Utils import *


class NsdKey:

   def __init__(self, name):
      self.name = name
      self.algorithm = ''
      self.secret = ''
      return

   def dump(self, ofile):
      if ofile == sys.stdout:
         print >> ofile, '=> NsdKey:'
         print >> ofile,'   name      = %s' % (self.name)
         print >> ofile,'   algorithm = %s' % (self.algorithm)
         print >> ofile,'   secret    = %s' % (self.secret)
      else:
         print >> ofile, 'key:'
         print >> ofile,'   name: "%s"' % (self.name)
         print >> ofile,'   algorithm: %s' % (self.algorithm)
         print >> ofile,'   secret: "%s"' % (self.secret)
      return

   def setName(self, val):
      self.name = val
      return

   def getName(self):
      return self.name

   def setAlgorithm(self, val):
      self.algorithm = val
      return

   def getAlgorithm(self):
      return self.algorithm

   def setSecret(self, val):
      self.secret = val
      return

   def getSecret(self):
      return self.secret

class NsdZone:

   def __init__(self, name, config, ipkeymap):
      self.name = name
      self.config = config
      self.include = ''
      self.zonefile = ''
      self.type = ''
      self.allow_notify = []		# empty unless we're a slave zone
      self.also_notify = []		# empty unless we're a master zone
      self.provide_xfr = []
      self.oldrootdir = ''
      self.ipkeymap = ipkeymap
      return

   def dump(self, ofile):
      if ofile == sys.stdout:
         print >> ofile, '=> NsdZone:'
         print >> ofile, '   name = %s' % (self.name)
         print >> ofile, '   include = %s' % (self.include)
         print >> ofile, '   zonefile = %s' % (self.zonefile)
         print >> ofile, '   type = %s' % (self.type)
         print >> ofile, '   allow_notify = %s' % (str(self.allow_notify))
         print >> ofile, '   also_notify = %s' % (str(self.also_notify))
         print >> ofile, '   provide_xfr = %s' % (str(self.provide_xfr))
      else:
         print >> ofile, ''
         print >> ofile, 'zone:'
         print >> ofile, '   name:        "%s"' % (self.name)
         print >> ofile, '   zonefile:    "%s"' % (self.zonefile)

	 # BOZO: this is a hack to avoid errors in the zone compiler
	 # and needs to be handled more intelligently; this allows the
	 # zone file to be non-existent (done by some ISPs) but still
	 # listed in the named.conf
	 if not os.path.exists(self.oldrootdir + self.zonefile):
	    report_error('? missing zone file "%s"' % \
	                 (self.oldrootdir + self.zonefile))

         nlist = self.allow_notify
	 if self.type == 'slave':
	    print >> ofile, '   # this is a slave zone.  masters are listed next.'
	    for ii in nlist:
	       print >> ofile, '   allow-notify: %s NOKEY' % (ii)
	       print >> ofile, '   request-xfr:  %s NOKEY' % (ii)
	 else:
            print >> ofile, '   include:     "%s"' % (self.include)
            for ii in self.also_notify:
	       print >> ofile, '   notify: %s NOKEY' % (ii)

	 for ii in self.provide_xfr:
	    if ii in self.ipkeymap:
	       print >> ofile, '   provide-xfr: %s %s' % (ii, self.ipkeymap[ii])
	       print >> ofile, '   notify: %s %s' % (ii, self.ipkeymap[ii])
	    else:
	       print >> ofile, '   provide-xfr: %s NOKEY' % (ii)
	       print >> ofile, '   notify: %s NOKEY' % (ii)

      return

   def setName(self, name):
      self.name = name
      return

   def getName(self):
      return self.name

   def setOldRootDir(self, name):
      self.oldrootdir = name
      return

   def getOldRootDir(self):
      return self.oldrootdir

   def setZonefile(self, file):
      self.zonefile = file
      return

   def getZonefile(self):
      return self.zonefile

   def setType(self, type):
      self.type = type
      return

   def getType(self):
      return self.type

   def addMaster(self, quad):
      self.masters.append(quad)
      return

   def getMasters(self):
      return self.masters

   def setInclude(self, name):
      self.include = name
      return

   def getInclude(self):
      return self.include

   def setAllowNotify(self, quad_list):
      self.allow_notify = quad_list
      return

   def getAllowNotify(self):
      return self.allow_notify

   def addAllowNotify(self, nlist):
      self.allow_notify.append(nlist)
      return

   def setAlsoNotify(self, quad_list):
      self.also_notify = quad_list
      return

   def getAlsoNotify(self):
      return self.also_notify

   def addAlsoNotify(self, nlist):
      self.also_notify.append(nlist)
      return

   def setProvideXfr(self, quad_list):
      self.provide_xfr = quad_list
      return

   def getProvideXfr(self):
      return self.provide_xfr

class NsdOptions:
   def __init__(self, config):
      self.database            = config.getValue('database')
      self.difffile            = config.getValue('difffile')
      self.identity            = config.getValue('identity')
      self.ip_address          = config.getValue('ip-address')
      self.logfile             = config.getValue('logfile')
      self.pidfile             = config.getValue('pidfile')
      self.port                = config.getValue('port')
      self.statistics          = config.getValue('statistics')
      self.username            = config.getValue('username')
      self.xfrd_reload_timeout = config.getValue('xfrd-reload-timeout')
      return

   def dump(self, ofile):
      if ofile == sys.stdout:
         print >> ofile, '=> Options:'
         print >> ofile, '   database:            %s' % (self.database)
         print >> ofile, '   difffile:            %s' % (self.difffile)
         print >> ofile, '   identity:            %s' % (self.identity)
         print >> ofile, '   ip-address:          %s' % (self.ip_address)
         print >> ofile, '   logfile:             %s' % (self.logfile)
         print >> ofile, '   pidfile:             %s' % (self.pidfile)
         print >> ofile, '   port:                %s' % (self.port)
         print >> ofile, '   statistics:          %s' % (self.statistics)
         print >> ofile, '   username:            %s' % (self.username)
         print >> ofile, '   xfrd-reload-timeout: %s' % (self.xfrd_reload_timeout)
      else:
         print >> ofile, '#'
         print >> ofile, '# All of the values below have been pulled from'
         print >> ofile, '# either defaults or the config file'
         print >> ofile, '#'
         print >> ofile, ''
         print >> ofile, 'server:'
         print >> ofile, '   database: %s' % (self.database)
         print >> ofile, '   difffile: %s' % (self.difffile)
         print >> ofile, '   identity: %s' % (self.identity)
	 iplist = self.ip_address.split()
	 for ii in iplist:
            print >> ofile, '   ip-address: %s' % (ii.strip())
         print >> ofile, '   logfile: %s' % (self.logfile)
         print >> ofile, '   pidfile: %s' % (self.pidfile)
         print >> ofile, '   port: %s' % (self.port)
         print >> ofile, '   statistics: %s' % (self.statistics)
         print >> ofile, '   username: %s' % (self.username)
         print >> ofile, '   xfrd-reload-timeout: %s' % \
	                 (self.xfrd_reload_timeout)
         print >> ofile, ''
      return

   def setDatabase(self, name):
      self.database = name
      return

   def getDatabase(self):
      return self.database

   def setDifffile(self, name):
      self.difffile = name
      return

   def getDifffile(self):
      return self.difffile

   def setIdentity(self, name):
      self.identity = name
      return

   def getIdentity(self):
      return self.identity

   def setIpAddress(self, quad):
      self.ip_address = quad
      return

   def getIpAddress(self):
      return self.ip_address

   def setLogfile(self, val):
      self.logfile = val
      return

   def getLogfile(self):
      return self.logfile

   def setPort(self, val):
      self.port = val
      return

   def getPort(self):
      return self.port

   def setPidfile(self, val):
      self.pidfile = val
      return

   def getPidfile(self):
      return self.pidfile

   def setStatistics(self, val):
      self.statistics = val
      return

   def getStatistics(self):
      return self.statistics

   def setXfrdReloadTimeout(self, val):
      self.xfrd_reload_timeout = val
      return

   def getXfrdReloadTimeout(self):
      return self.xfrd_reload_timeout

class NsdConf:

   def __init__(self, config):
      self.config = config
      self.fname = config.getValue('nsd_conf')
      self.files = config.getValue('nsd_files')
      self.preamble = config.getValue('nsd_preamble')
      self.acl_list = config.getValue('acl_list')
      self.oldrootdir = ''
      self.options = NsdOptions(config)
      self.includes = []
      self.zones = {}
      self.keys = {}
      self.ipkeymap = {}
      return

   def populate(self, named):
      self.oldrootdir = named.getOptions().getDirectory().replace('"','')
      if self.oldrootdir[len(self.oldrootdir)-1] != '/':
         self.oldrootdir += '/'

      klist = named.getKeys()
      for ii in klist:
         oldk = named.getKey(ii)
         k = NsdKey(oldk.getName())
         k.setAlgorithm(oldk.getAlgorithm())
         k.setSecret(oldk.getSecret())
	 self.keys[k.getName()] = k

	 #-- map each of the key ip addresses for faster lookup on dump()
	 iplist = oldk.getIpAddrs()
	 for jj in iplist:
	    if jj not in self.ipkeymap:
	       self.ipkeymap[jj] = k.getName()

      zlist = named.getZones()
      for ii in zlist:
         oldz = named.getZone(ii)
         z = NsdZone(oldz.getName(), self.config, self.ipkeymap)
	 z.setZonefile(oldz.getFile())
	 z.setInclude(self.acl_list)
	 z.setOldRootDir(self.oldrootdir)
	 if 'slave' in oldz.getType().split():
	    z.setType('slave')		# not used, but nice to know
	    nlist = oldz.getMasters()
	    for ii in nlist:
	       z.addAllowNotify(ii)
	    if len(oldz.getAllowNotify()) > 0:
	       nlist = oldz.getAllowNotify()
	    else:
	       nlist = named.getOptions().getAllowNotify()
	    for ii in nlist:
	       z.addAllowNotify(ii)
	 else:
	    z.setType('master')
	    if len(oldz.getAlsoNotify()):
	       nlist = oldz.getAlsoNotify()
	    else:
	       nlist = named.getOptions().getAlsoNotify()
	    for ii in nlist:
	       z.addAlsoNotify(ii)
	 if len(named.getOptions().getAllowTransfer()) > 0:
	    z.setProvideXfr(named.getOptions().getAllowTransfer())
	 self.zones[z.getName()] = z

      return

   def dump(self):
      report_info('=> NsdConf: dumping data from \"%s\"...' % (self.fname))
      self.options.dump(sys.stdout)
      report_info('=> NsdConf: list of includes...')
      report_info( str(self.includes))

      report_info('=> NsdConf: zone info...')
      zlist = self.zones.keys()
      zlist.sort()
      for ii in zlist:
         self.zones[ii].dump(sys.stdout)
   
      report_info('=> NsdConf: key info...')
      klist = self.keys.keys()
      klist.sort()
      for ii in klist():
         self.keys[ii].dump(sys.stdout)

      return

   def write_conf(self):
      nfile = open(self.fname, 'w+')
      self.options.dump(nfile)

      klist = self.keys.keys()
      klist.sort()
      for ii in klist:
         self.keys[ii].dump(nfile)

      zlist = self.zones.keys()
      zlist.sort()
      for ii in zlist:
         report_info('   writing info for zone "%s"...' % (self.zones[ii].getName()))
         self.zones[ii].dump(nfile)

      nfile.close()
      return
   
   def do_generate(self, newfd, start, stop, step, lhs, rrtype, rhs):
      #-- write the equivalent of the $GENERATE
      for ii in range(start, stop, step):
         left = lhs.replace('$', str(ii))
         right = rhs.replace('$', str(ii))
         print >> newfd, '%s %s %s' % (left, rrtype, right)
      return

   def make_zone_copy(self, oldpath, newpath):
      #-- copy the zone file line-by-line so we can catch $GENERATE usage
      report_info('=> copying "%s" to "%s"' % (oldpath, newpath))
      
      if not os.path.exists(oldpath):
         os.system('touch ' + newpath)
	 return

      oldfd = open(oldpath, 'r+')
      newfd = open(newpath, 'w+')
      line = oldfd.readline()
      while line:
         fields = line.split()
         if len(fields) > 0 and fields[0].strip() == '$GENERATE':
            report_info('=> processing "%s"...' % (line.strip()))
            index = 1

            #-- determine start/stop range
            indices = fields[index].split('-')
            start = 0
            stop  = 0
            if len(indices) == 2:
               start = int(indices[0])
               stop  = int(indices[1])
	       if stop < start:
                  bail('? stop < start in range "%s"' % (fields[1].strip()))
            else:
               bail('? invalid range "%s"' % (fields[1].strip()))
            index += 1
            
            #-- determine increment
            step = 1
            if len(fields) == 6:
               step = int(fields[index])
	       index += 1
      
            #-- get lhs
            lhs = fields[index].strip()
            index += 1
      
            #-- get RR type
            rrtype = fields[index].strip()
            if rrtype not in [ 'NS', 'PTR', 'A', 'AAAA', 'DNAME', 'CNAME' ]:
               bail('? illegal RR type "%s"' % (rrtype))
            index += 1
      
            #-- get rhs
            rhs = fields[index].strip()
            index += 1
      
	    #-- do the $GENERATE
            self.do_generate(newfd, start, stop, step, lhs, rrtype, rhs)
      
         else:		#-- just copy the line as is
	    print >> newfd, line,

         line = oldfd.readline()

      oldfd.close()
      newfd.close()
      
      return

   def write_zone_files(self):
      acls = {}

      zlist = self.zones.keys()
      zlist.sort()
      for ii in zlist:
         oldpath = self.oldrootdir + self.zones[ii].getZonefile()
	 newpath = self.config.getValue('tmpdir') + self.zones[ii].getZonefile()
	 if not os.path.exists(os.path.dirname(newpath)):
	    os.makedirs(os.path.dirname(newpath))
	 self.make_zone_copy(oldpath, newpath)

	 incl = self.zones[ii].getInclude()
         if acls.has_key(incl):
	    acls[incl] += 1
	 else:
	    acls[incl] = 1

      alist = acls.keys()
      alist.sort()
      for ii in alist:
	 newpath = self.config.getValue('tmpdir') + ii
         run_cmd('touch ' + newpath, 'touch "%s"' % (newpath))

      return

   def addZone(self, zone):
      name = zone.getName()
      self.zones[name] = zone
      return

   def getZones(self):
      return self.zones

   def addKey(self, key):
      self.keys[key.getName()] = key
      return

   def getKeys(self):
      return self.keys

   def getKey(self, name):
      if name in self.keys:
         return self.keys[name]
      else:
         return None




#!/usr/bin/env python
# Copyright (c) 2007, Secure64 Software Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#
#
#	class to tokenize a named.conf file
#
#

T_KEYWORD = 0
T_STRING  = 1
T_LBRACE  = 2
T_RBRACE  = 3
T_SEMI    = 4
T_COMMA   = 5
T_QUAD    = 6
T_COMMENT = 7
T_NAME    = 8
T_EOF     = 9
T_LPAREN  = 10
T_RPAREN  = 11

NAMED_KEYWORDS = [ 'algorithm', 'allow-notify', 'allow-recursion', 
                   'allow-transfer', 'also-notify',
                   'category', 'channel', 'check-names', 'controls', 
		   'coresize',
                   'directory', 'dump-file',
		   'file',
		   'IN', 'in', 'include',
		   'key', 'keys',
		   'logging',
		   'masters',
                   'options',
		   'pid-file', 'print-time',
		   'recursive-clients',
		   'secret', 'severity', 'statistics-file', 'syslog',
		   'transfer-format', 'type',
		   'version', 'versions',
		   'zone',
		 ]

class Tokenizer:

   def __init__(self, fname):
      infile = file(fname)
      self.data = ''.join(infile.readlines())
      self.curpos = 0
      self.lastpos = len(self.data)
      self.line = 1
      return

   def get(self):
      tok = ''
      done = False
      while not done:
         if self.curpos >= self.lastpos:
	    return (T_EOF, None, self.line)
         c = self.data[self.curpos]
	 if c == '\n':
	    self.line += 1
	 if c.isspace():
	    self.curpos += 1
	    continue
	 elif c == '{':
	    self.curpos += 1
	    return (T_LBRACE, c, self.line)
	 elif c == '}':
	    self.curpos += 1
	    return (T_RBRACE, c, self.line)
	 elif c == '(':
	    self.curpos += 1
	    return (T_LPAREN, c, self.line)
	 elif c == ')':
	    self.curpos += 1
	    return (T_RPAREN, c, self.line)
	 elif c == ';':
	    self.curpos += 1
	    return (T_SEMI, c, self.line)
	 elif c == '#':
	    while c != '\n':
	       tok += c
	       self.curpos += 1
	       c = self.data[self.curpos]
            self.curpos += 1
	    if c == '\n':
	       self.line += 1
	    return (T_COMMENT, tok, self.line)
	 elif c == '"':
	    tok += c
	    self.curpos += 1
	    c = self.data[self.curpos]
	    while c != '"':
	       tok += c
	       self.curpos += 1
	       c = self.data[self.curpos]
	    tok += c
            self.curpos += 1
	    return (T_STRING, tok, self.line)
	 elif c == '/' and self.data[self.curpos+1] == '/':
	    while c != '\n':
	       tok += c
	       self.curpos += 1
	       c = self.data[self.curpos]
	    if c == '\n':
	       self.line += 1
            self.curpos += 1
	    return (T_COMMENT, tok, self.line)
	 elif c == '/' and self.data[self.curpos+1] == '*':
	    cmtDone = False
	    tok += c
	    tok += self.data[self.curpos+1]
	    self.curpos += 2
	    c = self.data[self.curpos]
	    while not cmtDone:
	       if c == '*' and self.data[self.curpos+1] == '/':
		  tok += c
	          tok += self.data[self.curpos+1]
	          self.curpos += 2
	          return (T_COMMENT, tok, self.line)
	       else:
	          tok += c
		  if c == '\n':
		     self.line += 1
		  self.curpos +=1
	          c = self.data[self.curpos]
	    return (T_COMMENT, tok, self.line)
	 elif c.isdigit():
	    tok += c
	    self.curpos += 1
	    c = self.data[self.curpos]
	    while c.isdigit() or c == '.' or c == '/':
	       tok += c
	       self.curpos += 1
	       c = self.data[self.curpos]
	    return (T_QUAD, tok, self.line)
	 else:
	    while c.isalnum() or c == '_' or c == '-' or c == '.':
	       tok += c
	       self.curpos += 1
	       c = self.data[self.curpos]
	    if tok in NAMED_KEYWORDS:
	       return (T_KEYWORD, tok, self.line)
	    else:
	       return (T_NAME, tok, self.line)

      return

   def dump(self):
      print 'Tokenizer raw data:'
      print 'cur, last: %d, %d' % (self.curpos, self.lastpos)
      print self.data
      return

